m4_define([NW_VERSION], [1.0.1])
m4_define([NW_RELEASE_DATE], [2002/12/02 08:48 UTC])

AC_DEFUN([NW_VER_RELEASE_DATE],
[RELEASE_DATE='NW_RELEASE_DATE'

AC_SUBST([RELEASE_DATE])
])


AC_DEFUN([NW_VER_BUILD_DATE],
[BUILD_DATE=`$DATE_PROGRAM -u '+%Y/%m/%d %H:%M UTC' || echo unknown`

AC_SUBST([BUILD_DATE])
])


AC_DEFUN([NW_VER_BUILDER],
[
if test x$USER = x; then
    BUILDER=`$WHOAMI_PROGRAM || echo somebody`
else
    BUILDER=$USER
fi
if test x$HOST = x; then
    HOST=`$HOSTNAME_PROGRAM`
    if test x$HOST = x; then
        BUILDER=$BUILDER'@'`$UNAME_PROGRAM -n || echo unknown`
    else
        BUILDER=$BUILDER'@'$HOST
    fi
else
    BUILDER=$BUILDER'@'$HOST
fi

AC_SUBST([BUILDER])
])


dnl **************************************************************************
dnl GCC optimizations

AC_DEFUN([NW_ENABLE_GCC_OPTS],
[AC_ARG_ENABLE(gcc-optimizations,
    AC_HELP_STRING([--enable-gcc-optimizations],
        [enable GNU C compiler optimizations @<:@auto@:>@]),
    [nw_enable_gcc_opts=$enableval],
    [nw_enable_gcc_opts=$ac_cv_c_compiler_gnu])

AC_CACHE_CHECK([wether to enable GNU C compiler optimizations],
    [nw_cv_enable_gcc_opts],
    [nw_cv_enable_gcc_opts=$nw_enable_gcc_opts])

if test x$nw_cv_enable_gcc_opts = xyes; then
    nw_gcc_opts_fallback="-O3"
    nw_gcc_opts="-O3 -march=$target_cpu -mcpu=$target_cpu -funroll-loops"
    nw_gcc_opts_CFLAGS=`echo $CFLAGS | sed 's/^-O[[0-9]]/ &/' | sed 's/-O[[0-9]]$/& /' | sed 's/ -O[[0-9]] / /g'`

    AC_CACHE_CHECK([for optimizations],
        [nw_cv_gcc_opts],
        [CFLAGS=`echo "$nw_gcc_opts_CFLAGS $nw_gcc_opts" | sed 's/   */ /g'`
        AC_TRY_COMPILE([], [], [nw_cv_gcc_opts=$nw_gcc_opts], [nw_cv_gcc_opts=$nw_gcc_opts_fallback])])

    CFLAGS=`echo "$nw_gcc_opts_CFLAGS $nw_cv_gcc_opts" | sed 's/   */ /g'`
fi
])


dnl **************************************************************************
dnl Byte order

AC_DEFUN([NW_CHECK_BYTE_ORDER],
[AC_C_BIGENDIAN

if test x$ac_cv_c_bigendian = xyes; then
    nw_byte_order=big-endian
else
    nw_byte_order=little-endian
fi
m4_define([NW_BYTE_ORDER], [$nw_byte_order])
])


dnl **************************************************************************
dnl Use checksum by default

AC_DEFUN([NW_DEFAULT_CHECKSUM],
[AC_ARG_ENABLE(default-checksum,
    AC_HELP_STRING([--enable-default-checksum],
        [enable use of transmission checksum by default @<:@yes@:>@]),
    [nw_enable_def_checksum=$enableval],
    [nw_enable_def_checksum=yes])

AC_CACHE_CHECK([wether to enable checksum by default],
    [nw_cv_enable_def_checksum],
    [nw_cv_enable_def_checksum=$nw_enable_def_checksum])

if test x$nw_cv_enable_def_checksum = xyes; then
    nw_checksum_activation='by default'
    AC_DEFINE(USE_CHECKSUM_BY_DEFAULT, [1],
        [Define to 1 if you want to use transmission checksum by default.])
else
    nw_checksum_activation='on demand'
fi
])


dnl **************************************************************************
dnl Checksum algorithms

AC_DEFUN([NW_WITH_CHECKSUM],
[AC_CHECK_HEADER([mhash.h], [nw_mhash_h=yes], [nw_mhash_h=no])
AC_CHECK_LIB([mhash], [mhash_init], [nw_mhash_l=yes], [nw_mhash_l=no])
AC_CHECK_DECL([MHASH_MD5], [nw_mhash_md5=yes], [nw_mhash_md5=no], [#include <mhash.h>])
AC_CHECK_DECL([MHASH_RIPEMD160], [nw_mhash_rmd=yes], [nw_mhash_rmd=no], [#include <mhash.h>])
AC_CHECK_DECL([MHASH_SHA1], [nw_mhash_sha=yes], [nw_mhash_sha=no], [#include <mhash.h>])
AC_CHECK_DECL([MHASH_SHA256], [nw_mhash_sha256=yes], [nw_mhash_sha256=no], [#include <mhash.h>])
AC_CHECK_DECL([MHASH_TIGER], [nw_mhash_tiger=yes], [nw_mhash_tiger=no], [#include <mhash.h>])
AC_CHECK_DECL([MHASH_GOST], [nw_mhash_gost=yes], [nw_mhash_gost=no], [#include <mhash.h>])
AC_CACHE_CHECK([for mhash usability],
    [nw_cv_mhash],
    [if test x$nw_mhash_h = xyes && \
        test x$nw_mhash_l = xyes && \
        test x$nw_mhash_md5 = xyes && \
        test x$nw_mhash_rmd = xyes && \
        test x$nw_mhash_sha = xyes && \
        test x$nw_mhash_sha256 = xyes && \
        test x$nw_mhash_tiger = xyes && \
        test x$nw_mhash_gost = xyes; then
        nw_cv_mhash=yes
    else
        nw_cv_mhash=no
    fi
    ])

AC_CHECK_HEADER([openssl/evp.h], [nw_openssl_evp_h=yes], [nw_openssl_evp_h=no])
AC_CHECK_LIB([ssl], [EVP_DigestInit], [nw_openssl_evp_l=yes], [nw_openssl_evp_l=no])
if test x$nw_openssl_evp_l = xyes; then
    nw_openssl_evp_libs=-lssl
else
    AC_CHECK_LIB([ssl], [EVP_DigestUpdate], [nw_openssl_evp_l=yes],
                 [nw_openssl_evp_l=no], [-lcrypto])
    if test x$nw_openssl_evp_l = xyes; then
        nw_openssl_evp_libs="-lcrypto -lssl"
    fi
fi
AC_CACHE_CHECK([for openssl usability],
    [nw_cv_openssl_evp],
    [if test x$nw_openssl_evp_h = xyes && \
        test x$nw_openssl_evp_l = xyes; then
        nw_cv_openssl_evp=yes
    else
        nw_cv_openssl_evp=no
    fi
    ])

AC_ARG_WITH(checksum,
    AC_HELP_STRING([--with-checksum=<lib>],
        [select the library the checksum algorithms are taken from. <lib> can
        be `openssl', `mhash', `built-in' or `auto'. @<:@default is auto@:>@]),
    [case $withval in
    built-in|mhash|openssl|auto) nw_with_checksum=$withval ;;
    no) nw_with_checksum=built-in ;;
    *) nw_with_checksum=auto ;;
    esac
    ], [nw_with_checksum=auto])

AC_CACHE_CHECK([for checksum algorithm library],
    [nw_cv_with_checksum],
    [nw_checksum_lib=built-in
    
    case $nw_with_checksum in
    mhash)
        if test x$nw_cv_mhash = xyes; then
            nw_checksum_lib=mhash
        fi
        ;;
    openssl)
        if test x$nw_cv_openssl_evp = xyes; then
            nw_checksum_lib=openssl
        fi
        ;;
    built-in)
        ;;
    auto)
        if test x$nw_cv_openssl_evp = xyes; then
            nw_checksum_lib=openssl
        elif test x$nw_cv_mhash = xyes; then
            nw_checksum_lib=mhash
        fi
    esac

    nw_cv_with_checksum=$nw_checksum_lib])

case $nw_cv_with_checksum in
mhash)
    LIBS="-lmhash $LIBS"
    AC_DEFINE(USE_CHECKSUM_MHASH, [1],
        [Define to 1 if you want checksum functions from mhash.
         Exactly one of `USE_CHECKSUM_MHASH', `USE_CHECKSUM_OPENSSL', and
         `USE_CHECKSUM_BUILT_IN' must be defined.])
    
    ;;
openssl)
    LIBS="$nw_openssl_evp_libs $LIBS"
    AC_DEFINE(USE_CHECKSUM_OPENSSL, [1],
        [Define to 1 if you want checksum functions from openssl.
         Exactly one of `USE_CHECKSUM_MHASH', `USE_CHECKSUM_OPENSSL', and
         `USE_CHECKSUM_BUILT_IN' must be defined.])
    ;;
built-in)
    AC_DEFINE(USE_CHECKSUM_BUILT_IN, [1],
        [Define to 1 if you want built-in checksum functions.
         Exactly one of `USE_CHECKSUM_MHASH', `USE_CHECKSUM_OPENSSL', and
         `USE_CHECKSUM_BUILT_IN' must be defined.])
    ;;
esac

AM_CONDITIONAL(USE_CHECKSUM_MHASH, test x$nw_cv_with_checksum = xmhash)
AM_CONDITIONAL(USE_CHECKSUM_OPENSSL, test x$nw_cv_with_checksum = xopenssl)
AM_CONDITIONAL(USE_CHECKSUM_BUILT_IN, test x$nw_cv_with_checksum = xbuilt-in)
])


dnl **************************************************************************
dnl Summary

AC_DEFUN([NW_SUMMARY],
[AC_MSG_NOTICE([netrw tools summary

                  version   NW_VERSION
             release date   NW_RELEASE_DATE
               build date   ${BUILD_DATE}
                  builder   ${BUILDER}
                build for   ${target}
               byte order   NW_BYTE_ORDER

                   prefix   ${prefix}
      checksum activation   ${nw_checksum_activation}
       checksum functions   ${nw_cv_with_checksum}

                 CPPFLAGS   ${CPPFLAGS}
                   CFLAGS   ${CFLAGS}
                     LIBS   ${LIBS}
                  LDFLAGS   ${LDFLAGS}
])])

